pub enum DateTimeError {
InvalidMonth(u8),
InvalidDayOfMonth(u8),
InvalidHour(u8),
InvalidMinute(u8),
InvalidSecond(u8),
InvalidNanosecond(u32),
OutOfRange,
}Expand description
The error type returned when date-time components are invalid or correspond to a timestamp outside the representable range.
Variants
InvalidMonth(u8)
The month is not between 1 and 12.
InvalidDayOfMonth(u8)
The day of the month is less than 1, or more than the maximum value for this combination of year and month.
InvalidHour(u8)
The hour field value is not between 0 and 23.
InvalidMinute(u8)
The minute field value is not between 1 and 59.
InvalidSecond(u8)
The second field value is not between 1 and 59.
InvalidNanosecond(u32)
The nanosecond field value is more than 999 999 999.
OutOfRange
This date-time value cannot be represented as a TAI timestamp with the specified epoch.