Enum tai_time::ParseDateTimeError
source · pub enum ParseDateTimeError {
InvalidFieldValue,
InvalidFieldWidth,
MissingField,
RangeError(DateTimeError),
}Expand description
The error type returned when a date-time string is invalid or corresponds to a timestamp outside the representable range.
Variants
InvalidFieldValue
A field value is either not of the expected numeric type or is out of range for the expected numeric type.
InvalidFieldWidth
The width of a fixed-width or minimum-width field is invalid.
MissingField
A field is missing.
RangeError(DateTimeError)
One of the field value is out of its expected range, or the corresponding timestamp is outside the representable range.